/*
 * Copyright (c) 2009 Anthony Steed and Manuel Fradinho Oliveira
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the 
 * following conditions are met:
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the 
 *    following disclaimer.
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the 
 *    following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of 
 *    this software must display the following acknowledgement:
 *       This product includes software from the book "Networked 
 *       Graphics" written by Anthony Steed and Manuel Fradinho
 *       Oliveira published by Morgan Kaufmann
 * 4. Neither the name of the University nor of the Department
 *    may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 5. Neither the name of Cyntelix may be used to enforese or 
 *    promote products derived from this software without prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO 
 * EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION). HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact Info: Anthony Steed
 *               a.steed@cs.ucl.ac.uk
 *
 *               Manuel Fradinho Oliveira
 *               mfradinho@cyntelix.com
 *
 */
import java.util.*;
import javax.vecmath.*;
import java.net.*;

public class ProtocolP2PMaster extends ProtocolServer {

  public ProtocolP2PMaster () {
    super();
  }

  public StringBuffer code(Flock flock) {
    StringBuffer auxStr = null;
    _strBuf.delete(0, _strBuf.length());
    switch (_state) {
        case ACTIVE  :  _strBuf.append(flock._id);
                        _strBuf.append('#');
                        Enumeration auxFlock = flock.getBoids();
                        _strBuf.append(auxFlock.nextElement().toString());
                        for (;auxFlock.hasMoreElements();) {
                          _strBuf.append('%');
                          _strBuf.append(auxFlock.nextElement().toString());
                        }
                        auxStr = _strBuf;
                        break;
        case INACTIVE : auxStr = prepareAnnounceStr(flock);
                        auxStr.append(((UDPNetwork)_networkRef)._receiveSocket.getLocalAddress().getHostAddress());
                        auxStr.append('#');
                        auxStr.append(((UDPNetwork)_networkRef)._receiveSocket.getLocalPort());
                        DatagramPacket packet = new DatagramPacket(auxStr.toString().getBytes(),auxStr.length());
                        try {
                          packet.setAddress(_serverIP);
                          packet.setPort(_serverPort);
                          ((UDPNetwork)_networkRef)._sendSocket.send(packet);
                        } catch (Exception e) {
                          e.printStackTrace();
                        }
                        _state = ACTIVE;
                        auxStr = null;
                        break;
    }
    return auxStr;
  }

  public void decode(String str) {
    String auxID = null;
    Boid boid = null;
    int index = 0;
    int id = 0;
    FlockRemote auxFlock = null;
    Flock auxFlockRef = null;
    String ip;

    index = str.indexOf('!');
    if (index >= 0) {
      str = str.substring(index+1);
      index = str.indexOf('#');
      auxID = str.substring(0,index);
      int flockID = Integer.parseInt(auxID);
      str = str.substring(index+1);
      index = str.indexOf('#');
      auxID = str.substring(0,index);
      int red = Integer.parseInt(auxID);
      str = str.substring(index+1);
      index = str.indexOf('#');
      auxID = str.substring(0,index);
      int green = Integer.parseInt(auxID);
      str = str.substring(index+1);
      index = str.indexOf('#');
      auxID = str.substring(0,index);
      int blue = Integer.parseInt(auxID);
      str = str.substring(index+1);
      index = str.indexOf('#');
      ip = str.substring(0,index);
      str = str.substring(index+1);
      int port = Integer.parseInt(str.trim());
      auxFlock = new FlockRemoteFlag(_datamodel._numBoids, new Color3f(red, green, blue));
      auxFlock.setID(flockID);
      auxFlock.setPort(port);
      auxFlock.setIP(ip);
      _datamodel.addRemoteFlock(auxFlock);
      if (_serverPort < 0) {
         try {
           StringBuffer auxStr2 = null;
           Flock auxflock = _datamodel.getLocalFlock();
           auxStr2 = prepareAnnounceStr(auxflock);
           auxStr2.append(((UDPNetwork)_networkRef)._receiveSocket.getLocalAddress().toString().substring(1));
           auxStr2.append('#');
           auxStr2.append(((UDPNetwork)_networkRef)._receiveSocket.getLocalPort());
           DatagramPacket packet = new DatagramPacket(auxStr2.toString().getBytes(),auxStr2.length());
           packet.setAddress(InetAddress.getByName(ip));
           packet.setPort(port);
           ((UDPNetwork)_networkRef)._sendSocket.send(packet);
           FlockRemote auxflockRemote;
           for (Enumeration e=_datamodel.getRemoteFlocks();e.hasMoreElements();) {
             auxStr2.delete(0, auxStr2.length());
             auxflockRemote = (FlockRemote)e.nextElement();
             if (flockID != auxflockRemote._id) {
               auxStr2 = prepareAnnounceStr(auxflockRemote);
               packet = new DatagramPacket(auxStr2.toString().getBytes(),auxStr2.length());
               packet.setAddress(((UDPNetwork)_networkRef)._receiveSocket.getLocalAddress());
               packet.setPort(port);
               ((UDPNetwork)_networkRef)._packet.setPort(auxflockRemote.getPort());
               ((UDPNetwork)_networkRef)._packet.setAddress(auxflockRemote.getIP());
               ((UDPNetwork)_networkRef)._sendSocket.send(packet);
               ((UDPNetwork)_networkRef)._sendSocket.send(((UDPNetwork)_networkRef)._packet);
             }
           }
         } catch (Exception e) {
           e.printStackTrace();
         }
      }
      _rendererRef.addFlock(auxFlock);
    } else {
      index = str.indexOf('#');
      auxID = str.substring(0,index);
      id = Integer.parseInt(auxID);
      auxFlockRef = _datamodel.getRemoteFlock(id);
      if (auxFlockRef != null) {
          str = str.substring(index+1);
          index = str.indexOf('%');
          for (Enumeration iterator = auxFlockRef.getBoids(); iterator.hasMoreElements();) {
            boid = (Boid)iterator.nextElement();
            if (index < 0) break;
            boid.unpackString(str.substring(0,index));
            str = str.substring(index+1);
            index = str.indexOf('%');
          }
          boid.unpackString(str);
      }
    }
  }

  protected StringBuffer prepareAnnounceStr(Flock flock) {
     StringBuffer str = new StringBuffer();
     str.append('!');
     str.append(flock._id);
     str.append("#");
     int red = (int)flock._colour.getX();
     int green = (int)flock._colour.getY();
     int blue = (int)flock._colour.getZ();
     str.append(red);
     str.append("#");
     str.append(green);
     str.append("#");
     str.append(blue);
     str.append("#");
     return str;
  }

  protected StringBuffer prepareAnnounceStr(FlockRemote flock) {
     StringBuffer str = prepareAnnounceStr((Flock)flock);
     str.append(flock.getIPString());
     str.append('#');
     str.append(flock.getPort());
     return str;
  }
}
