/*
 * Copyright (c) 2009 Anthony Steed and Manuel Fradinho Oliveira
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the 
 * following conditions are met:
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the 
 *    following disclaimer.
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the 
 *    following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of 
 *    this software must display the following acknowledgement:
 *       This product includes software from the book "Networked 
 *       Graphics" written by Anthony Steed and Manuel Fradinho
 *       Oliveira published by Morgan Kaufmann
 * 4. Neither the name of the University nor of the Department
 *    may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 5. Neither the name of Cyntelix may be used to enforese or 
 *    promote products derived from this software without prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO 
 * EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION). HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact Info: Anthony Steed
 *               a.steed@cs.ucl.ac.uk
 *
 *               Manuel Fradinho Oliveira
 *               mfradinho@cyntelix.com
 *
 */
import javax.swing.*;
import java.awt.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.behaviors.vp.*;
import java.util.*;

public class Renderer extends JFrame implements IUniverseBuilder {
  private static final int BOUNDSIZE = 1024;
  UniversePanel            _panel;
  BoundingSphere           _bounds;
  TransformGroup           _flockRef;
  Properties               _config;
  FlockBehaviour           _behaviour;
  Network                  _networkRef;
  IBoids                   _boidsRef;

  public Renderer(){
      Container c = getContentPane();
      c.setLayout(new BorderLayout());
      _panel = new UniversePanel();

      _flockRef = new TransformGroup();

      _panel.set(this);
      c.add(_panel, BorderLayout.CENTER);
      setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
      _networkRef = null;
  }
  
  public void setup() {
    _panel.create();
    pack();
    setResizable(false);  
    setVisible(true);  
  }

  public void setNetwork(Network network) {
    _networkRef = network;
  }
  
  public BranchGroup createUniverse() {
    BranchGroup group = new BranchGroup();
    _bounds = new BoundingSphere(new Point3d(0,0,0), BOUNDSIZE);
    return group;
  }
  
  public void createLighting(BranchGroup scene) {
    Color3f white = new Color3f(1.0f, 1.0f, 1.0f);
    AmbientLight ambientLightNode = new AmbientLight(white);
    ambientLightNode.setInfluencingBounds(_bounds);
    scene.addChild(ambientLightNode);

    Vector3f light1Direction  = new Vector3f(-0.0f, -0.0f, -2.0f);
    Vector3f light2Direction  = new Vector3f(-0.5f, -0.8f, -2.0f);

    DirectionalLight light1 = new DirectionalLight(white, light1Direction);
    light1.setInfluencingBounds(_bounds);
    scene.addChild(light1);

    DirectionalLight light2 = new DirectionalLight(white, light2Direction);
    light2.setInfluencingBounds(_bounds);
    scene.addChild(light2);  
  }
  
  public void createBackground(BranchGroup scene) {
    Background back = new Background();
    back.setApplicationBounds(_bounds);
    back.setColor(0.17f, 0.65f, 0.92f);    // sky colour
    scene.addChild(back);
  }
  
  public void addContent(BranchGroup scene) {
    _behaviour = new FlockBehaviour();
    _flockRef.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    _flockRef.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    _flockRef.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
    scene.addChild(_flockRef);
    scene.addChild(_behaviour);
    _behaviour.setNetwork(_networkRef);
    Flock localFlock = _boidsRef.getLocalFlock();
    BranchGroup localFlockBG = createFlockBG(localFlock);
    _behaviour.setLocalFlock(localFlock);
    _flockRef.addChild(localFlockBG);
  }

  public void setBoids(IBoids boids) {
    _boidsRef = boids;
  }

  public void positionUser() {
    Point3d USERPOSN = new Point3d(0,30,50);
  
    TransformGroup steerTG = _panel.getViewer();
    Transform3D t3d = new Transform3D();
    steerTG.getTransform(t3d);
    t3d.lookAt( USERPOSN, new Point3d(0,0,0), new Vector3d(0,1,0));
    t3d.invert();
    steerTG.setTransform(t3d);    
    OrbitBehavior orbit = new OrbitBehavior(_panel._canvas, OrbitBehavior.REVERSE_ALL);
    orbit.setSchedulingBounds(_bounds);
    _panel.getViewPlatform().setViewPlatformBehavior(orbit);  
  }

  protected BranchGroup createFlockBG(Flock flock) {
    BranchGroup auxBG = new BranchGroup();  
    Boid boid = null;    
   Transform3D offset = new Transform3D();
    for (Enumeration e = flock.getBoids(); e.hasMoreElements();) {
      boid = (Boid)e.nextElement();
      offset.setTranslation(boid._position);
      boid.setTransform(offset);
      auxBG.addChild(boid);
    }    
    auxBG.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    return auxBG;
  }

  public void addFlock(Flock flock) {
    BranchGroup auxFlockBG = createFlockBG(flock);
    _flockRef.addChild(auxFlockBG);
    _behaviour.addFlock(flock);
  }

}
